@@ -4,12 +4,12 @@ use File::Path 'mkpath';
 use File::Copy 'copy';
 use Config;
 
-my $VERSION = '1.05';			# Changelog at end
+my $VERSION = '1.06';			# Changelog at end
 die "Debugging cycle detected"		# set to -1 to allow extra iteration
   if ++$ENV{PERL_DEBUG_MCODE_CYCLE} > 1;
 
 my %opt;
-$opt{$1} = shift while ($ARGV[0] || 0) =~ /^-([dq1OB])$/;
+$opt{$1} = shift while ($ARGV[0] || 0) =~ /^-([dq1OBU])$/;
 if ($opt{1}) {
   open STDERR, '>&STDOUT' or warn "can't redirect STDERR to STDOUT";
 } else {
@@ -20,7 +20,7 @@ my $bd = (my $bd0 = 'dbg-bld') . ($opt{O} || '');
 @ARGV >= 1 or die <<EOP;
 
 Usage:
- $0 [-B] [-d] [-q] [-1] [-O] check-module [failing-script1 failing-script2 ...]
+ $0 [-B] [-U] [-d] [-q] [-1] [-O] check-module [failing-script1 failing-script2 ...]
 
 A tool to simplify remote debugging of build problems for XSUB modules.
 By default, output goes to STDERR (to pass through the test suite wrappers).
@@ -42,9 +42,10 @@ debugging for FAILING-SCRIPTs is done.
 Options:	With -d, prefers dbx to gdb (DEFAULT: prefer gdb).
 		With -q and no FAILING-SCRIPTs, won't print anything unless a
 			failure of loading is detected.
-		With -1, all output goes to STDOUT.
+		With -1, all our output goes to STDOUT.
 		With -O, makes a non-debugging build.
 		With -B, builds in a subdirectory even if no debugger was found.
+		With -U will reuse the build directory if present.
 
 Assumptions:
   Should be run in the root of a distribution, or its immediate subdir.
@@ -161,53 +162,59 @@ if ($opt{B}) {
   die $postpone if $postpone;
 }
 
-die "Directory $bd exist; won't overwrite" if -d $bd;
-mkdir $bd or die "mkdir $bd: $!";
+my $build_was_OK = -f "$bd/autodebug-make-ok";
+die "Directory $bd exist; won't overwrite" if -d $bd and not ($opt{U} and $build_was_OK);
+mkdir $bd or die "mkdir $bd: $!" unless -d $bd;
 chdir $bd or die "chdir $bd: $!";
 
-open MF, '../MANIFEST' or die "Can't read MANIFEST: $!";
-while (<MF>) {
-  next unless /^\S/;
-  s/\s.*//;
-  my ($f, $d) = m[^((.*/)?.*)];
-  -d $d or mkpath $d if defined $d;	# croak()s itself
-  copy "../$f", $f or die "copy `../$f' to `$f' (inside $bd): $!";
-}
-close MF or die "Can't close MANIFEST: $!";
-
-my(@extraflags, $more, $subst) = 'OPTIMIZE=-g';
-# Work around bugs in Config: 'ccflags' may contain (parts???) of 'optimize'.
-if ($opt{O}) {			# Do not change debugging
-  @extraflags = ();
-} elsif ($Config{ccflags} =~ s/(?<!\S)\Q$Config{optimize}\E(?!\S)//) {
-  # e.g., Strawberry Perl
-  $subst++;
-} elsif ($Config{gccversion} or $Config{cc} =~ /\b\w?cc\b/i) {	# assume cc-flavor
-  #     http://www.cpantesters.org/cpan/report/ef2ee424-1c8e-11e6-b928-8293027c4940
-  #     http://www.cpantesters.org/cpan/report/4837b230-1d9d-11e6-91cb-6b7bc172c7fc
-  # Extra check:
-  $more++ if $Config{optimize} =~ /(?<!\S)-O(\d*|[a-z]?)(?!\S)/;
-}
-if ($more or $subst) {
-  my $FL;
-  $subst++ if ($FL = $Config{ccflags}) =~ s/(?<!\S)-(s|O(\d*|[a-z]?)|fomit-frame-pointer)(?!\S)//g;
-  push @extraflags, qq(CCFLAGS=$FL) if $subst;
-  for my $f (qw(ldflags lddlflags)) {
-    push @extraflags, qq(\U$f\E=$FL)
-      if ($FL = $Config{$f}) =~ s/(?<!\S)-s(?!\S)//g;
+sub do_subdir_build () {
+  open MF, '../MANIFEST' or die "Can't read MANIFEST: $!";
+  while (<MF>) {
+    next unless /^\S/;
+    s/\s.*//;
+    my ($f, $d) = m[^((.*/)?.*)];
+    -d $d or mkpath $d if defined $d;	# croak()s itself
+    copy "../$f", $f or die "copy `../$f' to `$f' (inside $bd): $!";
   }
+  close MF or die "Can't close MANIFEST: $!";
+
+  my(@extraflags, $more, $subst) = 'OPTIMIZE=-g';
+  # Work around bugs in Config: 'ccflags' may contain (parts???) of 'optimize'.
+  if ($opt{O}) {			# Do not change debugging
+    @extraflags = ();
+  } elsif ($Config{ccflags} =~ s/(?<!\S)\Q$Config{optimize}\E(?!\S)//) {
+    # e.g., Strawberry Perl
+    $subst++;
+  } elsif ($Config{gccversion} or $Config{cc} =~ /\b\w?cc\b/i) {	# assume cc-flavor
+    #     http://www.cpantesters.org/cpan/report/ef2ee424-1c8e-11e6-b928-8293027c4940
+    #     http://www.cpantesters.org/cpan/report/4837b230-1d9d-11e6-91cb-6b7bc172c7fc
+    # Extra check:
+    $more++ if $Config{optimize} =~ /(?<!\S)-O(\d*|[a-z]?)(?!\S)/;
+  }
+  if ($more or $subst) {
+    my $FL;
+    $subst++ if ($FL = $Config{ccflags}) =~ s/(?<!\S)-(s|O(\d*|[a-z]?)|fomit-frame-pointer)(?!\S)//g;
+    push @extraflags, qq(CCFLAGS=$FL) if $subst;
+    for my $f (qw(ldflags lddlflags)) {
+      push @extraflags, qq(\U$f\E=$FL)
+        if ($FL = $Config{$f}) =~ s/(?<!\S)-s(?!\S)//g;
+    }
+  }
+
+  system $^X, 'Makefile.PL', @extraflags and die "system(Makefile.PL @extraflags): rc=$?";
+  my $make = $Config{make};
+  $make = 'make' unless defined $make;
+  system $make and die "system($make): rc=$?";
+  { open my $f, '>', 'autodebug-make-ok'; }	# Leave a footprint of a successful build
 }
 
-system $^X, 'Makefile.PL', @extraflags and die "system(Makefile.PL @extraflags): rc=$?";
-my $make = $Config{make};
-$make = 'make' unless defined $make;
-system $make and die "system($make): rc=$?";
+do_subdir_build() unless -f 'autodebug-make-ok';
 
 die $postpone if $postpone;	# Reached without a debugger only with -B
 
 my $p = ($^X =~ m([\\/]) ? $^X : `which perl`) || $^X;
 chomp $p unless $p eq $^X;
-my(@cmd, $ver, $ver_done);
+my(@cmd, $ver, $ver_done, $dscript);
 
 for my $script (@ARGV) {
   $script = "../$script" if not -f $script and -f "../$script";
@@ -229,7 +236,7 @@ EOP
 info w32 thread-information-block
 echo \\n=====================================\\n\\n
 EOE
-    print TT <<EOP;		# Slightly different order than dbx...
+    print TT ($dscript = <<EOP);		# Slightly different order than dbx...
 run -Mblib $script
 echo \\n=====================================\\n\\n
 bt
@@ -294,7 +301,7 @@ EOP
 script print "??? info w32 thread-information-block"
 script print "\\n=====================================\\n"
 EOE
-    print TT <<EOP;		# Slightly different order than dbx...
+    print TT ($dscript = <<EOP);		# Slightly different order than dbx...
 run -Mblib $script
 script print "\\n=====================================\\n"
 bt
@@ -335,9 +342,9 @@ EOP
 	    qq(run -Mblib $script; echo; echo =================================; echo; where -v; echo; echo =================================; echo; dump; echo; echo =================================; echo; regs; echo; echo =================================; echo; list -i +1; echo; echo =================================; echo; list -i -10; echo; echo =================================; echo; echo ============== up 1:; up; dump; echo; echo ============== up 2:; up; dump; echo; echo ============== up 3:; up; dump; echo; echo ============== up 4:; up; dump; echo ==============; /usr/proc/bin/pmap -F \$proc; quit),
 	    $p);
   }
-  warn "Debugger's version: $ver" unless $ver_done++;
-  warn 'Running {{{', join('}}} {{{', @cmd), '}}}';
-  system @cmd and die "Running @cmd: rc=$?";
+  warn "\nDebugger's version: $ver\n" unless $ver_done++;
+  warn 'Running {{{', join('}}} {{{', @cmd), "}}}\n\n";
+  system @cmd and die "Running @cmd: rc=$?", ($dscript ? "\n========= script begin\n$dscript\n========= script end\n\t" : '');
 }
 1;
 
@@ -373,3 +380,6 @@ __END__
 	Debugger's version was reported too late (after a possible die()!).
 	Were emitting a wrong section from Makefile's.
 	Emit MakeMaker's ARGV from Makefile too.
+1.06	Report the script if the debugger failed.
+	Leave ./autodebug-make-ok in the build directory if make was successful.
+	New option -U for using an existing directory with a successful build.
